; All these sources/objects are (c) Simon Brattel 1991

		include symbols.inc	; Define them

Cur_FR		equ 15			; Frames

UDK_nKeys	equ 12			; This many keys (+ 0)

CODE		SEGMENT public 		;

		ASSUME  CS:CODE		;

		db "(c) Simon Brattel"

Start:		mov ax,StackSeg		; get Stack Segment Origin
		mov SS,ax		; and initialize the SS register
		mov SP,o TOS		; set SP to Top Of Stack

		; Drive the parameters

		call Drive_Params	; Drive them

		mov al,cs:fForce_Ints	; Flags to the int. handler
		mov ah,cs:fKill_FL	;

		call Set_Frame_Flags	; Set them

		; Free surplus RAM

		call Deallocate		; Release it

		; Set the interrupts

		in al,Int_Data		; Read the interupt mask
		mov CS:Int_Mask,al	; Save it

		mov al,0FCh		; DI all except timer and kbd
		out Int_Data,al		;

		call SetUpDisplay	; Make EGA
		jc Exit			; Exit

		mov cs:VDU_Seg,ax	; Save the segment
		mov cs:FRONT_Seg,ax	; Save the segment

		; Init the game

		call far ptr Init_Games	; Start the game code
		jc Exit1		; Exit

		; We are ready, show things

		call Play_Games		; Do it

		; Exit

Exit:		call far ptr End_Games	; Clear it

Exit1:		call Kill_VDU		; Reset display mode

		mov al,CS:Int_Mask	; Unmask interrupts
		out Int_Data,al		;

		; Right, try to save any data ?

		call Write_Files	; Write them

		; The exit message

		mov ax,TEXT		; Use DS
		mov es,ax		;

		mov bx,o Exit_Msg	; Show this
		call Dos_Print_Str	; Show it

		mov ah,terminate	;
		int dos			; Done

	; Message

	db "Hackers, please don't alter anything !"

; Link the frame-lock routine

F_Frame_Lock:	call Next_Frame		; Lock it
		retf			; Done (FAR)

; Write asciiz at es:bx to dos

Dos_Print_Str:	mov dl,es:[bx]		; Get the char
		cmp dl,0		; End ?
		jz DPS_X		; Exit

		inc bx			; Skip the char

		mov ah,Display_Char	; Display it
		int dos			; Show it

		jmp s Dos_Print_Str	; Loop

DPS_X:		ret			; Done

	db "There are some hidden bits you would destroy"

; Release the unused RAM

Deallocate:	mov bx,o TOS		; Point at top of ss

		mov cl,4		; Convert to paragraphs
		shr bx,cl		;
		inc bx			; add one for luck

		mov cx,ss		; Get stack segment
		add bx,cx		; Add stack segment
		
		mov cx,es		; Get extra segment
		sub bx,cx		; Subtract it

		; bx contains no. of paragraphs from es to the top of ss

		mov ah,Set_Block	; Deallocate surplus RAM
		int dos			;

		mov al,9		; "Internal memory"
		ret			;

		; Now we include the sources

		include front.inc	; The front-end code

		include utils.inc	; Some code

		include options.inc	; The game options

		include params.inc	; Command-line parameters

		include files.inc	; Save/Load files

		include keys.inc	; Key code

Int_Mask	db ?			; 8259A state

VDU_Seg		dw ?			; VDU segment
FRONT_Seg	dw ?			; VDU segment for 640_350

		extrn Char_Set		:near	; 256 colour score chars

CODE		ends

		extrn Init_Games	:far
		extrn End_Games		:far
		extrn Play_Game		:far

		extrn SetUpDisplay	:near
		extrn Link_Kbd		:near
		extrn Unlink_Kbd	:near
		extrn Kill_VDU		:near
		extrn Rand_32		:near

		extrn Set_640_350	:near
		extrn Set_320_200	:near

		extrn Next_Frame	:near
		extrn Set_Frame_Flags	:near
		extrn Long_Time_F	:near

		extrn Key_Matrix	:abs
		extrn Seed		:abs

		; Export bits

		public F_Link_Kbd	; Kbd driver
		public F_Unlink_Kbd	;

		public F_Frame_Lock	;

		public F_Read_File	;
		public F_Write_File	;

		public Test_Up		; Key stuff
		public Test_Down	;
		public Test_Left	;
		public Test_Right	;
		public Test_Fire	;
		public Test_Pause	;
		public Test_Boss	;
		public Test_Esc		;

		public Test_F1		;
		public Test_F2		;
		public Test_F3		;
		public Test_F4		;
		public Test_F5		;
		public Test_F6		;
		public Test_F7		;
		public Test_F8		;
		public Test_F9		;
		public Test_F10		;
		public Test_F11		;
		public Test_F12		;

		public Test_UA		;
		public Test_DA		;

; Data

FONT		SEGMENT word public	; A segment for the fonts

		; The character sets

		extrn Char_Set_1:near	; Large
		extrn Char_Set_2:near	; Medium
		extrn Char_Set_3:near	; Small

		; The design-design logo

		extrn Logo		:near

FONT		ends			; End it

; More data

TEXT		SEGMENT word public	; A segment for the fonts

		; The essential text

		extrn Ess_Text		:near

		; The playing text

		extrn Play_Text		:near

		; The various text

		extrn Var_Text		:near

		; The ordering text

		extrn Ord_Text		:near

		; The extra options text

		extrn Extra_Text	:near

		; The story text

		extrn Story_Text	:near

		; The shareware notice

		extrn Share_Text	:near

		; The exit message

Exit_Msg	db 5 dup (cCret,cLF)

		db "Design-Design - you'll love it . . . it's a way of life."
		db 3 dup (cCret,cLF)

		db 0

TEXT		ends			; End it

DATA		SEGMENT word public	; Data seg

		include hiscores.inc	; The highscore table

		; The names of the keys

Key_Chars	db 00h			; 00
		db 80h			; 01 (ESC)
		db "1"			; 02
		db "2"			; 03
		db "3"			; 04
		db "4"			; 05
		db "5"			; 06
		db "6"			; 07
		db "7"			; 08
		db "8"			; 09
		db "9"			; 0A
		db "0"			; 0B
		db "-"			; 0C
		db "="			; 0D
		db 81h			; 0E (DEL)
		db 82h			; 0F (TAB)

		db "Q"			; 10
		db "W"			; 11
		db "E"			; 12
		db "R"			; 13
		db "T"			; 14
		db "Y"			; 15
		db "U"			; 16
		db "I"			; 17
		db "O"			; 18
		db "P"			; 19
		db "["			; 1A
		db "]"			; 1B
		db 83h			; 1C (CRET)
		db 84h			; 1D (CTRL)
		db "A"			; 1E
		db "S"			; 1F

		db "D"			; 20
		db "F"			; 21
		db "G"			; 22
		db "H"			; 23
		db "J"			; 24
		db "K"			; 25
		db "L"			; 26
		db ";"			; 27
		db "'"			; 28
		db "`"			; 29
		db 85h			; 2A (L-SHIFT)
		db "#"			; 2B
		db "Z"			; 2C
		db "X"			; 2D
		db "C"			; 2E
		db "V"			; 2F

		db "B"			; 30
		db "N"			; 31
		db "M"			; 32
		db ","			; 33
		db "."			; 34
		db "/"			; 35
		db 86h			; 36 (R-SHIFT)
		db 00h			; 37
		db 87h			; 38 (ALT)
		db 88h			; 39 (SPACE)
		db 89h			; 3A (CAPS)
		db 8Ah			; 3B (F1)
		db 8Bh			; 3C (F2)
		db 8Ch			; 3D (F3)
		db 8Dh			; 3E (F4)
		db 8Eh			; 3F (F5)

		db 8Fh			; 40 (F6)
		db 90h			; 41 (F7)
		db 91h			; 42 (F8)
		db 92h			; 43 (F9)
		db 93h			; 44 (F10)
		db 94h			; 45 (NUM-LOCK)
		db 95h			; 46 (SCROLL-LOCK)
		db 96h			; 47 (HOME)
		db 97h			; 48 (UP)
		db 98h			; 49 (PG_UP)
		db 99h			; 4A (NUM -)
		db 9Ah			; 4B (LEFT)
		db 9Bh			; 4C (NUM-5)
		db 9Ch			; 4D (RIGHT)
		db 9Dh			; 4E (NUM +)
		db 9Eh			; 4F (END)

		db 9Fh			; 50 (DOWN)
		db 0A0h			; 51 (PG_DOWN)
		db 0A1h			; 52 (INS)
		db 0A2h			; 53 (DEL)
		db 00h			; 54
		db 00h			; 55
		db 00h			; 56
		db 0A3h			; 57 (F11)
		db 0A4h			; 58 (F12)
		db 00h			; 59
		db 00h			; 5A
		db 00h			; 5B
		db 00h			; 5C
		db 00h			; 5D
		db 00h			; 5E
		db 00h			; 5F

		db 00h			; 60
		db 00h			; 61
		db 00h			; 62
		db 00h			; 63
		db 00h			; 64
		db 00h			; 65
		db 00h			; 66
		db 00h			; 67
		db 00h			; 68
		db 00h			; 69
		db 00h			; 6A
		db 00h			; 6B
		db 00h			; 6C
		db 00h			; 6D
		db 00h			; 6E
		db 00h			; 6F

		db 00h			; 70
		db 00h			; 71
		db 00h			; 72
		db 00h			; 73
		db 00h			; 74
		db 00h			; 75
		db 00h			; 76
		db 00h			; 77
		db 00h			; 78
		db 00h			; 79
		db 00h			; 7A
		db 00h			; 7B
		db 00h			; 7C
		db 00h			; 7D
		db 00h			; 7E
		db 00h			; 7F

		; Extended strings

Key_Strings	db "(ESC)",0		; 80
		db "(BS)",0		; 81
		db "(TAB)",0		; 82
		db "(CRET)",0		; 83
		db "(CTRL)",0		; 84
		db "(Left SHIFT)",0	; 85
		db "(Right SHIFT)",0	; 86
		db "(ALT)",0		; 87
		db "(SPACE)",0		; 88
		db "(CAPS)",0		; 89
		db "(F1)",0		; 8A
		db "(F2)",0		; 8B
		db "(F3)",0		; 8C
		db "(F4)",0		; 8D
		db "(F5)",0		; 8E
		db "(F6)",0		; 8F
		db "(F7)",0		; 90
		db "(F8)",0		; 91
		db "(F9)",0		; 92
		db "(F10)",0		; 93
		db "(NUM LOCK)",0	; 94
		db "(SCROLL LOCK)",0	; 95
		db "(HOME)",0		; 96
		db "(UP)",0		; 97
		db "(PgUp)",0		; 98
		db "(-)",0		; 99
		db "(LEFT)",0		; 9A
		db "(5)",0		; 9B
		db "(RIGHT)",0		; 9C
		db "(+)",0		; 9D
		db "(END)",0		; 9E
		db "(DOWN)",0		; 9F
		db "(PgDn)",0		; A0
		db "(INS)",0		; A1
		db "(DEL)",0		; A2
		db "(F11)",0		; A3
		db "(F12)",0		; A4

		; Now, the list of sprite areas

;		align 2			; Even

;pMemory_List	dw ?			; Pointer to the memory block list

;Memory_List	dw 257 dup (0)		; List here

		; Now, we have the keystore for the action-replay

		align 2			; Even

pKey_Data	dw ?			; Pointer to the replay list

Old_Key_Flags	dw ?			; Old states
Key_Counts	dw ?			; Time since last change

		; The boss-screen is imported

		extrn Boss_Screen	:near

DATA		ends

V_MASK		segment byte

		db 320*200 dup (?)

V_MASK		ends

CODE1		segment public
CODE1		ends

StackSeg	segment stack

		dw 4000 dup (?)
TOS		equ this word

StackSeg	ends

		end Start

