; Show the logo on the current screen

Logo_Width	equ 608/8;544/8		; Width of the logo

Show_Logo:	mov si,o Logo		; Point at it

		; Now, setup the EGA/VGA registers

		call Set_EGA_Regs	; Set them

		mov ax,FONT		; FONT segment
		mov ds,ax		;

		mov ax,cs:VDU_Seg	; Display segment
		mov es,ax		;

		mov di,0		; Point at the screen
		mov bl,1		; Colour

		; Now, calculate the screen address

		mov dx,3CEh		; Graphics controller
		mov al,08h		; Point at the BIT-MASK register

		mov ch,0		; Count := 0

		mov w cs:Logo_Cnt,350	; Set the count

		mov bh,0AAh		; Mask

ShL_LpY:	lea bp,Logo_Width[di]	; Point at the end of the line

		; Now, get the next byte

ShL_Lp:		mov cl,ds:[si]		; Get the next byte
		inc si			; Skip

		cmp cl,0C0h		; Count, or byte ?
		jnc ShL_Cnt		; Count, skip

		; Byte, dump it

		mov ah,cl		; Set the bit-map
		and ah,bh		; Invert it
		out dx,ax		; Write it

		; Colour

		and b es:[di],bl	; Write it
		inc di			;

		; End of line ?

		cmp bp,di		; End ?
		jne ShL_Lp		; No, loop

		; EOL

		jmp s ShL_EOL		; Skip

	; cx = count

ShL_Cnt:	and cl,03Fh		; Setup the count

		mov ah,ds:[si]		; Get the next byte
		inc si			; Skip

		and ah,bh		; Invert it
		out dx,ax		; Write the bit-map

ShL_Lp1:	and b es:[di],bl	; Set the colour
		inc di			; Skip

		loop ShL_Lp1		; Loop

		; Is this the end of the line ?

		cmp di,bp		; EOL ?
		jne ShL_Lp		; No, loop

		; Right, move down

ShL_EOL:	add di,Y_Step_640-Logo_Width ; Move down one line

		; Alternate mask

		not bh			; Flip it

		; Is this the end ?
		
		dec w cs:Logo_Cnt	; End ?
		jnz ShL_LpY		; No, loop

		; Yes, done

		ret			; Done
		
; Show a screen-full of text from DATA:si

; cx = X dx = Y ah = Colour

; Exits with al=1A if reached the end

Print_Screen:	mov bx,TEXT		; TEXT data segment
		mov ds,bx		;

		mov al,ds:[si]		; Get the next char
		cmp al,01Ah		; EOT ?
		je PSC_End		; Yes, exit

		inc si			; Skip this character

		cmp al,"#"		; Control char ?
		je PSC_CTL		; Yes, exit

		cmp al,cLF		; LF ?
		je Print_Screen		; Yes, ignore it

		cmp al,cCret		; Cret ?
		je PSC_Cret		; Yes, skip

		; Print this character

		call Print_Char		; Print it

		; Loop

		jmp s Print_Screen	; Loop

	; Exit case

PSC_End:	ret			; Done

	; Cret

PSC_Cret:	add dx,cs:PS_DX		; Step down

		mov cx,cs:PS_LHS	; LHS

		jmp s Print_Screen	; Loop

	; Drive the control char

PSC_CTL:	mov al,ds:[si]		; Get the control char
		inc si			;

		cmp al,"E"		; End of screen ?
		je PSC_End		; Yes, exit

		cmp al,"X"		; X position
		je PSC_X		;

		cmp al,"D"		; X step
		je PSC_DX		;

		cmp al,"Y"		; Y position
		je PSC_Y		;

		cmp al,"L"		; LHS
		je PSC_LHS		;

		cmp al,"F"		; Font
		je PSC_FONT		;

		cmp al,"C"		; Colour
		jne Print_Screen	; No, ignore

		; Get the colour

		call PSC_Num		; Giz a number

		mov ah,al		; Colour

		jmp s Print_Screen	; Done

	; X

PSC_X:		push ax			; Save

		call PSC_Num		; Giz a number

		mov cx,ax		; X

		pop ax			; Restore

		jmp s Print_Screen	; Done

	; DX

PSC_DX:		push ax			; Save

		call PSC_Num		; Giz a number

		mov cs:PS_DX,ax		; DX

		pop ax			; Restore

		jmp s Print_Screen	; Done

	; Y

PSC_Y:		push ax			; Save

		call PSC_Num		; Giz a number

		mov dx,ax		; Y

		pop ax			; Restore

		jmp s Print_Screen	; Done

	; LHS

PSC_LHS:	push ax			; Save

		call PSC_Num		; Giz a number

		mov cs:PS_LHS,ax	; DX

		pop ax			; Restore

		jmp s Print_Screen	; Done

	; FONT

PSC_FONT:	push ax			; Save

		call PSC_Num		; Giz a number

		mov cs:Curr_CS,al	; CS

		pop ax			; Restore

		jmp Print_Screen	; Done

; Giz a 16-bit decimal

PSC_Num:	push dx			; Save

		call ReadDec32		; Get a number

		pop dx			; Restore
		ret			; Done

ReadDec32:	push bx			; Save bx, result in dx,ax
		push cx			; 

		call SkipSpc		; Find no.
		jz RD_Fail		; None

		call DecNo		; Valid digit ?
		jb RD_Fail		; None

		cbw			; ah := 0
		mov bx,ax		; bx,dx = accumulator
		mov dx,0		;

RDLp:		inc si			; Step to next char

		call DecNo		; Valid ?
		jb RD_Done		; End of number

		cbw			; Save digit
		push ax			;

		mov ax,bx		; * 1
		mov cx,dx		; 

		shl bx,1		; Shift left
		rcl dx,1		;
		jc RD_Fail1		; Overflow

		shl bx,1		; Shift left
		rcl dx,1		;
		jc RD_Fail1		; Overflow

		add bx,ax		; * 5
		adc dx,cx		;
		jc RD_Fail1		; Overflow

		add bx,bx		;
		adc dx,dx		;
		jc RD_Fail1		; * 10

		pop ax			; Restore digit

		add bx,ax		; Result
		adc dx,0		;

		jmp RDLp		; Next digit

RD_Fail1:	pop ax			; Lose digit
		mov al,2		; Overflow error

		jmp RD_Fail2		;

RD_Fail:	mov al,1		; No number error

RD_Fail2:	pop cx			; Restore regs
		pop bx			;

		stc			; Fail
		ret			;

RD_Done:	mov ax,bx		; Get low word

		pop cx			;
		pop bx			;

		clc			;
		ret			;

DecNo:		mov al,ds:[si]		; Get character
		sub al,'0'		;
		jb DecNoOut		; Not decimal

		cmp al,10		; 0 .. 9
		cmc			; CF if below
DecNoout:	ret			;

Skip1Spc:	inc si			; Skip one

SkipSpc:	mov al,ds:[si]		; Read char

		cmp al,00		; End ?
		je SS_X			; Yes, exit

		cmp al," "		; Space ?
		jz Skip1Spc		; Yes, strip it

		cmp al,cTab		; Tab ?
		jz Skip1Spc		; Yes, strip it

		cmp al,cNull		; Is it EOL ?
SS_X:		ret			;

; Drive the sounds, wrecks al

Set_Freq:	push ax			; Save

		mov al,182		; Setup 8253
		out 43h,al		; Drive it

		jmp s $+2		; Delay
		jmp s $+2		; Delay
		jmp s $+2		; Delay

		pop ax			; Restore

		out 42h,al		; LSB

		jmp s $+2		; Delay
		jmp s $+2		; Delay
		jmp s $+2		; Delay

		xchg al,ah		; MSB
		out 42h,al		;

		xchg al,ah		; Restore

		ret			; Done

; Speaker on

Spk_On:		test b cs:Noise_F,-1	; To bleep, or not to bleep
		jz Spk_X		; No, exit

		in al,61h		; Get PPA
		jmp s $+2		; Delay
		jmp s $+2		; Delay
		jmp s $+2		; Delay

		or al,03h		; Drive it
		out 61h,al		; Set it

Spk_X:		ret			; Done
				
; Speaker off

Spk_Off:	in al,61h		; Get PPA
		jmp s $+2		; Delay
		jmp s $+2		; Delay
		jmp s $+2		; Delay

		and al,0FDh		; Drive it
		out 61h,al		; Set it

		ret			; Done

; Set the displayed video page from al = 0, 1

Set_VDU_Page:	push dx			; Save
		push bx			;
		push ax			;

		; Set one of two possible video pages

		mov bx,0		; Page 0

		cmp al,0		; 0 ?
		je SVP_Set		; Yes, set it

		mov bx,((Y_Step_640 * 350) or 15) + 1	; A segment boundary

SVP_Set:	mov dx,3D4h		; CRTC

		mov al,0Dh		; Low word
		mov ah,bl		;

		out dx,ax		; Set it

		jmp $+2			; Pause

		mov al,0Ch		; Low word
		mov ah,bh		;

		out dx,ax		; Set it

		; Done, exit		

		pop ax			; Restore
		pop bx			;
		pop dx			;

		ret			; Done		

; Set the drawing video page from al = 0, 1

Set_DRAW_Page:	push dx			; Save
		push bx			;
		push ax			;

		; Set one of two possible video pages

		mov bx,0A000h		; Page 0

		cmp al,0		; 0 ?
		je SDP_Set		; Yes, set it

		mov bx,0A000h + (((Y_Step_640 * 350) or 15) + 1)/16	; A segment boundary

SDP_Set:	mov cs:VDU_Seg,bx	; Set it

		; Done, exit		

		pop ax			; Restore
		pop bx			;
		pop dx			;

		ret			; Done		

; Clear the screen

CLS:		push ds			; Save
		push di			;
		push dx			;
		push cx			;
		push ax			;

		; Setup the registers

		call Set_EGA_Regs	; Setup for the write

		mov ax,0FF08h		; Set the bit-mask
		out dx,ax		;

		; Now, clear the holding latches

		mov ds,cs:VDU_Seg	; Point at it
		mov di,0		;

		; Now just perform the writes

		mov cx,640/8*350	; Length (one screens)
		mov al,0		; Value

CLS_Lp:		and [di],al		; Clear it
		inc di			; Step

		loop CLS_Lp		; Loop

		; Done, restore

		pop ax			; Restore
		pop cx			;
		pop dx			;
		pop di			;
		pop ds			;

		ret			; Done

; Clear the 320 by 200 screen

CLS_320:	push es			; Save
		push di			;
		push dx			;
		push cx			;
		push ax			;

		; Now, clear the 256 colour display

		mov es,cs:VDU_Seg	; Point at it
		mov di,0		;

		; Now just perform the writes

		mov cx,320*200		; Width

		mov al,0		; Value

		rep stosb		; Clear it

		; Done, restore

		pop ax			; Restore
		pop cx			;
		pop dx			;
		pop di			;
		pop es			;

		ret			; Done

; Copy the screen, from current to background

Duplicate_Page:	push ds			; Save
		push es			;
		push di			;
		push si			;
		push cx			;
		push ax			;

		mov ax,cs:VDU_Seg	; Save it
		push ax			;

		; Setup the registers

		call Set_EGA_Regs	; Setup for the write

 		mov ax,00905h		; Mode/Reg Write mode 1
		out dx,ax		; Set it

		; Now, clear the holding latches

		mov ax,0A000h		; Page 0
		mov ds,ax		;
		mov si,0		;

		mov ax,0A000h + (((Y_Step_640 * 350) or 15) + 1)/16 ; Page 1
		mov es,ax		;
		mov di,0		;

		; Is page 0 the current one ?

		test b cs:Front_Screen,1 ; Copy 0 to 1 ?
		jz DP_01		; Yes

		push ds			; Other way round
		push es			;
		pop ds			;
		pop es			;

		; Now just perform the writes

DP_01:		mov cx,640/8*350	; Length (both screens)

		rep movsb		; Copy it

		; Done, restore

		pop ax			; Restore
		mov cs:VDU_Seg,ax	;

		pop ax			; Restore
		pop cx			;
		pop si			;
		pop di			;
		pop es			;
		pop ds			;

		ret			; Done

; Given a pointer in si, test for exit keys and leave
; cx = time out value, returns key = FF

Drive_Menu:	call Test_Key		; Get a key
		jz DM_TO		; Not found

		; Get the key

		mov al,cs:Curr_Key	; Get the key

		mov b cs:Key_Waiting,0	; Clear the flag

		; Force this to upper-case

		call Make_Upper		; Force it to upper-case

		; Is it used in this menu ?

DM_Lp:		mov bl,cs:[si]		; Get key

		cmp bl,0		; End ?
		jz DM_X			; Yes, skip

		cmp bl,al		; This one ?
		jz DM_Found		; Yes, skip

		; Not pressed, skip

		add si,3		; Skip to next

		jmp s DM_Lp		; Loop

	; Found, exit

DM_Found:	mov bx,cs:1[si]		; Get exit vector

		clc			; OK
		ret			;

	; No key, check time-out

DM_TO:		loop DM_X		; Exit if not yet

		; Return a key value FF

		mov al,0FFh		; Key-code = FF

		jmp s DM_Lp		; Loop

	; Not found, fail

DM_X:		stc			; Fail
		ret			;

; Force al to upper-case

Make_Upper:	cmp al,"z"		; a .. z
		ja MU_X			; No

		cmp al,"a"		; a .. z ?
		jb MU_X			; No

		add al,"A"-"a"		; Make lc

MU_X:		ret			; Done

; Wait for the pause key

Pause:		test b cs:Key_Matrix+1,1; Test for escape
		jz Pause		; Loop

Pause1:		test b cs:Key_Matrix+1,1; Test for escape
		jnz Pause1		; Loop

		ret			; Done

; Print a string of ASCIIZ cs:[si] at cx,dl using colours based on ah

Print_Str:	push ds			; Save

		push cs			; Use CS
		pop ds			;

		call Print_Str_DS	; Show it

		pop ds			; Restore
		ret			; Done

; Print a string of ASCIIZ ds:[si] at cx,dl using colours based on ah

Print_Str_DS:	mov al,ds:[si]		; Get char
		inc si			; Skip

		cmp al,0		; End ?
		jz PS_X			; Yes, exit

		cmp al,1		; Colour ?
		jz PS_Col		; Yes, skip

		cmp al,2		; Posn ?
		jz PS_Pos		; Yes, skip

		cmp al,3		; CS ?
		jz PS_CS		; Yes, skip

		; Must be printable, print it

		call Print_Char		; Show it

		jmp s Print_Str_DS	; Loop

	; Get its character-set

PS_CS:		mov al,ds:[si]		; Chars
		inc si			;

		mov cs:Curr_CS,al	; Store it

		jmp s Print_Str_DS	; Loop

	; Get its colour

PS_Col:		mov ah,ds:[si]		; Colour
		inc si			;

		jmp s Print_Str_DS	; Loop

	; Get a position

PS_Pos:		mov cx,ds:[si]		; X
		inc si			;
		inc si			;

		mov dx,ds:[si]		; Y
		inc si			;
		inc si			;

		jmp s Print_Str_DS	; Loop

PS_X:		ret			; Done

vPC_Fail:	mov b cs:PC_Width,6	; Default

vPC_Exit:	jmp PC_Exit		; Vector it

; Print a character AL with base colour AH at cx = X, dx = Y

Print_Char:	push ax			; Save

		push es			; Save
		push ds			;

		push si			;
		push di			;

		push dx			;
		push cx			;
		push bx			;

		mov cs:PC_Col,ah	; Save colour

		mov cs:PC_X,cx		; X
		mov cs:PC_Y,dx		; Y

		; Calc char set

		mov si,o Char_Set_1	; Large

		test b cs:Curr_CS,-1	; CS 0 ?
		jz vPC_CS		; Yes, skip

		mov si,o Char_Set_2	; Medium

		cmp b cs:Curr_CS,1	; CS 1 ?
		je vPC_CS		; Yes, skip

		mov si,o Char_Set_3	; Small

		; Calc char address

vPC_CS:		sub al," "		; Align
		jc vPC_Fail		; Can't, fail

		cmp al,128-" "		; 20 .. 7F
		ja vPC_Fail		; No, skip

		mov bl,al		;
		mov bh,0		; 16-bit

		add bx,bx		; * 2
		add bx,bx		; * 4

		add bx,si		; Point at it

		; Set the segment

		mov ax,FONT		; Set the segment
		mov es,ax		;

		; Get data for this char

		mov ax,es:[bx]		; Get the Height/Width
		mov w cs:PC_Height,ax	; Store dY/dX

		cmp ax,0		; Not a valid char ?
		je vPC_Exit		; No, exit

		mov bx,es:2[bx]		; Point at the char
		xchg bl,bh		; Motorola format
		add si,bx		; Point at it

		; Now, setup the EGA/VGA registers

		call Set_EGA_Regs	; Set them

		; Now, calculate the screen address

		; Now, calc the display ptr, clip value, offset etc

		mov ax,cs:PC_Y		; ax  := Y

		mov bx,Y_Step_640;cs:VDU_Width	; Display YStep

		mul bx			; ax := Address fn(Y)

		mov di,ax		; Into di

		; Now, look at X, first the rotate value

		mov al,cl		; Bottom bits
		and al,07h		; Clip

		mov cs:PC_Rot,al	; Store it

		; Now the screen ptr

		sar cx,1		; / 8
		sar cx,1		;
		sar cx,1		;

		; cx = offset in bytes

		add di,cx		; Add to the screen ptr

		cmp cx,Y_Step_640;cs:VDU_Width	; Are we on the screen ?
		jae PC_Exit		; No, skip

		; Now, draw the char

		mov ds,cs:VDU_Seg	; ds:di = dest

		mov al,cs:PC_Width	; Width in bytes
		add al,7		;

		shr al,1		; / 8
		shr al,1		;
		shr al,1		;

		mov cs:PC_iX_Cnt,al	; Save the count
		
		; Show it

		mov cl,cs:PC_Rot	; Rotate count
		mov ch,cs:PC_Col	; Colour

		mov dx,3CEh		; Graphics controller
		mov al,08h		; Point at the BIT-MASK register

		; Print the character

PC_Y_Lp:	mov bp,di		; Save ptr

		; Setup X_Cnt

		mov ah,cs:PC_iX_Cnt	; Get it
		mov cs:PC_X_Cnt,ah	; Save it

		mov bl,0		; No bits rotated yet

		; Get the next byte

PC_X_Lp:	mov bh,es:[si]		; Get the next byte
		inc si			; Skip

		mov ah,bl		; Old bits
		mov bl,0		; Clear the new ones

		ror bx,cl		; Align

		or ah,bh		; Combine

		; Set the bit-mask

		out dx,ax		; Set the mask port

		; Set the colour

		and ds:[di],ch		; Colour
		inc di			; Skip

		; Now loop across X

		dec b cs:PC_X_Cnt	; Loop
		jnz PC_X_Lp		;

		; Remainder of shifted char

		mov ah,bl		; Old bits

		; Set the bit-mask

		out dx,ax		; Set the mask port

		; Set the colour

		and ds:[di],ch		; Colour

		; Now, step down

		lea di,Y_Step_640[bp] ;cs:VDU_Width	; Next line
;		add di,bp		;

		; Loop

	 	dec b cs:PC_Height	; Loop
		jnz PC_Y_Lp		;

		; Done, exit

PC_Exit:	mov al,cs:PC_Width	; Width
		cbw			; 16-bit

		pop bx			;
		pop cx			;
		pop dx			;

		pop di			;
		pop si			;

		add cx,ax		; Skip this char

		pop ds			;
		pop es			;

		pop ax			;
		ret			; Done

; Set the EGA/VGA registers for a write operation

Set_EGA_Regs:	; Set Write mode 2, Read-mode 1

		mov dx,3CEh		; Graphics controller

 		mov ax,00A05h		; Mode/Reg
		out dx,ax		; Set it

		; Set the colour don't care register

		mov ax,00007h		; CDC := 00
		out dx,ax		;

		; Set the function to replace

		mov ax,00003h		; DRF := 0
		out dx,ax		;

		; Done, end

		ret			; Done

; Any 'keys' waiting ?

Test_Key:	push ax			; Save acc

		; Is there a char waiting ?

		test b cs:Key_Waiting,1 ; Do we have a key ?
		jnz TK_Yes		; Yes, return it

		; Any key-strokes ?

		mov ah,1		; Test if key ready
		int Kbd			;

		jnz TK_Decode		; We have a key-stroke, decode it

		pop ax			; Z, no keys
		ret			;

TK_Yes:		xor ax,ax		; Z
		inc ax			; NZ

		pop ax			; Return key ready
		ret			;

; We have an IBM key-stroke, we must now decide what to do with it

TK_Decode:	push bx			; Save 'em
		push cx			;
		push dx			; ax is already stacked

		; Get the key-stroke

		mov ah,0		; Read a key (wait)
		int kbd			;

		; Convert IBM 16-bit format to Para-Sys 8-bit format

		cmp al,0		; Special key ?
		jnz TKD_1		; No, got its value

		mov bx,o Key_Tab-2	; Convert it

TKD_Lp:		inc bx			; Step ptr
		inc bx			;

		cmp b cs:[bx],0		; End of table ?
		jz TKD_NF		; Not found

		cmp cs:[bx],ah		; This key ?
		jne TKD_Lp		; No, loop

		mov al,cs:1[bx]		; Convert it

		; al = Key value

TKD_1:		mov b cs:Curr_Key,al	; Store the key

		mov b cs:Key_Waiting,1	; Set the flag

TKD_2:		pop dx			; Restore them
		pop cx			;
		pop bx			;
		pop ax			;

		jmp Test_Key		; Try again

	; Show unknown key

TKD_NF:		mov al,ah		; Key code

;		call WriteHex8		; Show it

		jmp s TKD_2		; Exit

; Look-up the internal key codes

Key_Tab:	db 48h,cUp		; Up

		db 4Bh,cLeft		; Left
		db 73h,cicLeft		; Left (ctrl)

		db 4Dh,cRight		; Right
		db 74h,cicRight		; Right(ctrl)

		db 50h,cDown		; Down

		db 47h,ciHome		; Home
		db 77h,cicHome		; Ctl Home

		db 49h,ciPgUp		; PgUp
		db 84h,cicPgUp		; Ctl PgUp

		db 4Fh,ciEnd		; End
		db 75h,cicEnd		; Ctl End

		db 51h,ciPgDn		; PgDn
		db 76h,cicPgDn		; Ctl PgDn

		; Misc

		db 52h,cInsMode		; Ins
		db 53h,cRem		; Del

		db 0Fh,cisTab		; Shift-Tab

		db 3Bh,cCLS		; Function Keys
		db 3Ch,cClear_Line	;
		db 3Dh,cClear_To_End	;
		db 3Eh," "		;
		db 3Fh," "		;
		db 40h," "		;
		db 41h,cAttr0		;
		db 42h,cAttr1		;
		db 43h,cAttr2		;
		db 44h,cAttr3		;

		db 54h,80h		; Shifted function keys
		db 55h,81h		;
		db 56h,82h		;
		db 57h,83h		;
		db 58h,84h		;
		db 59h,85h		;
		db 5Ah,86h		;
		db 5Bh,87h		;
		db 5Ch,88h		;
		db 5Dh,89h		;

		db 5Eh,90h		; Alt function keys
		db 5Fh,91h		;
		db 60h,92h		;
		db 61h,93h		;
		db 62h,94h		;
		db 63h,95h		;
		db 64h,96h		;
		db 65h,97h		;
		db 66h,98h		;
		db 67h,99h		;

		db 68h,0A0h		; Ctrl function keys
		db 69h,0A1h		;
		db 6Ah,0A2h		;
		db 6Bh,0A3h		;
		db 6Ch,0A4h		;
		db 6Dh,0A5h		;
		db 6Eh,0A6h		;
		db 6Fh,0A7h		;
		db 70h,0A8h		;
		db 71h,0A9h		;

		db 0			; End

; Get a key, without a cursor

Get_Key:	push bx			; Save regs
		push ax			;

		push dx			;
		push cx			;

		push ds			; Save 

		; Now wait for a key . . .

GK_Lp:		call Test_Key		; Got one ?
		jz GK_Lp		; No, loop

		; Now we must get the next (buffered) 'key'

		mov al,cs:Curr_Key	; Get the key

		mov b cs:Key_Waiting,0	; Clear the flag

		pop ds			; Done

		pop cx			;
		pop dx			;

		pop bx			; Get ah
		mov ah,bh		;

		pop bx			; Get bx

		ret			; Done

; Notice that the cursor can only sensibly be driven by Get_Key since it is
; a flag to the user telling them when the system can accept keystrokes.

Get_Key_Cur:	push bx			; Save regs
		push ax			;

		push dx			;
		push cx			;

		push ds			; Save 

		; Enable the cursor

		call Cur_On		; Assert cursor

		; Setup cursor flash

		mov bh,1		; State
		mov bl,Cur_FR		; Flash-rate

		; Now wait for a key . . .

GKC_Lp:		call Next_Frame		; Synchronise to the video

		dec bl			; Time to change the cursor ?
		jnz GKC_1		; No, skip

		; Yes, flash it

		mov bl,Cur_FR		; Restore timer

		xor bh,1		; Flip the state
		jne GKC_FC_1		; To on

		; Turn it off

		call Cur_Off		; Turn it off

		jmp s GKC_1		; Test for a key

	; Turn it on

GKC_FC_1:	call Cur_On		; Turn it on

		; Test for a key

GKC_1:		call Test_Key		; Got one ?
		jz GKC_Lp		; No, loop

		; Now we must get the next (buffered) 'key'

		mov al,cs:Curr_Key	; Get the key

		mov b cs:Key_Waiting,0	; Clear the flag

		; Clear the cursor

		call Cur_Off		; Negate cur

		pop ds			; Restore

		pop cx			;
		pop dx			;

		pop bx			; Get ah
		mov ah,bh		;

		pop bx			; Get bx

		ret			; Done

; Drive the cursor around

Cur_On:		push cx			; Save
		push ax			;

		; Show the cursor

		mov al,"_"		; Cursor
		call Print_Char		; Show it

		pop ax			;
		pop cx			;
		ret			;

Cur_Off:	push cx			; Save
		push ax			;

		; Hide the cursor

		mov al,"_"		; Cursor
		mov ah,0		; Colour = black

		call Print_Char		; Show it

		pop ax			;
		pop cx			;
		ret			;

; Write the number

WriteDec16:	push ax			; Save regs
		push bx			;
		push di			;

		mov di,ax		; Shuffle regs

		mov bx,o PowersOfTen	; Digit values table

		mov ah,' '		; To be displayed for 0

WD_Lp:		call WD_CalcDigit	; Calc digit

		cmp al,0		; Zero ?
		jnz WD_1		; No

		mov al,ah		; Print ' ' or '0'
		call Dec_Char		;

		jmp s WD_Next		; Do next digit

WD_1:		mov ah,'0'		; Use 0 in future
		
		or al,ah		; Make char
		call Dec_Char		;

WD_Next:	add bx,2		; Next power of ten

		cmp w cs:[bx],1		; Last power of ten ?
		jnz WD_Lp		; No, continue

		call WD_CalcDigit	; Calc digit

		or al,'0'		; Make char
		call Dec_Char		;

		pop di			; Restore
		pop bx			;
		pop ax			;

		ret			; Done
		
WD_CalcDigit:	mov al,-1		; Initial digit
	
WDCD_Lp:	inc al			; Inc digit

		sub di,CS:[bx]		; 16-bit

		jnc WDCD_Lp		; Try again

		add di,CS:[bx]		; Step back

		ret			; Done						

; Decimal char out

Dec_Char:	push ax			; Save

		mov ah,cs:Dec_Col	; Colour

		call Print_Char		; Show a char

		pop ax			; Restore
		ret			; Done

; Get time/date string into buffer (ds:bx)

Read_Clock:	push ds			; Save

		mov ax,DATA		; Use DATA seg
		mov ds,ax		;

		mov bx,o Clock_Buff	; Buffer

		push di			; Save
		push bx			;

		mov ah,Get_Time		; Call DOS
		int dos			;

		add bx,10		; Point at hours

		mov dl,"a"		; Assume "am"

		mov al,ch		; Hours
		cmp al,0		; not 12 ?
		jnz RC_1		; No, try next

		mov al,12		; Yes, 12
		jmp RC_Hours		; Show

RC_1:		cmp al,12		; am ?
		jb RC_Hours		; Yes, show

		mov dl,"p"		; "pm"
		jz RC_Hours		; 12 ?

		sub al,12		; 1..11

RC_Hours:	call RC_Show		; Display

		mov b [bx],":"		; Sep
		inc bx			; Step

		mov al,cl		; Minutes
		call RC_Show		; Display

		mov b [bx],":"		; Sep
		inc bx			; Step

		mov al,dh		; Seconds
		call RC_Show		; Display

		mov b [bx]," "		; Sep
		inc bx			; Step

		mov [bx],dl		; am or pm
		inc bx			;
		mov b [bx],"m"		;
		inc bx			;

		mov b [bx]," "		; Sep
		inc bx			;

		mov ah,Get_Date		; Giz date
		int dos			;

		push ax			; Save day of week

		mov al,dl		; Day
		call RC_Show		;

		mov b [bx],"/"		; Sep
		inc bx			;

		mov al,dh		; Month
		call RC_Show		;

		mov b [bx],"/"		; Sep
		inc bx			;

		mov ax,cx		; Year
		call RC_Show_Year	;

;		mov b [bx],0		; Terminate

		pop cx			; Restore day

		mov di,o sDays-1	; Point at day strings

RC_Day_Lp:	inc di			; Step past term

		cmp cl,0		; This one ?
		jz RC_Day		; Yes

		dec cl			; Step day

RCD_Lp:		cmp b cs:[di],0		; Skip to next
		jz RC_Day_Lp		; At end

		inc di			; Skip

		jmp RCD_Lp		; Loop

RC_Day:		pop bx			; Get ptr
		push bx			;

RCD_Lp2:	mov al,cs:[di]		; Get day name
		cmp al,0		; Done ?
		jz RC_Done		; Yes

		inc di			;

		mov [bx],al		; Store
		inc bx			;

		jmp RCD_Lp2		; Loop

RC_Done:	pop bx			; Restore
		pop di			;
		pop ds			;
		ret			; Done

; Show ax (1980 .. 2099) as ascii

RC_Show_Year:	sub ax,1900		; 80 .. 199

		cmp al,100		; Out of range ?
		jb RC_Show		; This centuary

		sub al,100		; Next centuary

		jmp RC_Show		;

; Show al (0..99) as ascii

RC_Show:	push ax			; Save

		aam			; Get digits

		add ah,"0"		; Tens
		mov [bx],ah		;
		inc bx			;

		add al,"0"		; Units
		mov [bx],al		;
		inc bx			;

		pop ax			; Restore
		ret			; Done

sDays:		db "Sunday    ",0	; Data for read clock
		db "Monday    ",0	;
		db "Tuesday   ",0	;
		db "Wednesday ",0	;
		db "Thursday  ",0	;
		db "Friday    ",0	;
		db "Saturday  ",0	;

; Fixed

PowersOfTen	dw 10000,1000,100,10,1	; Powers for decimal print

; Vars

Logo_Cnt	dw 0			; Logo line count

Dec_Col		db 0			; Decimal routine colour

Key_Waiting	db 0			; There is a key waiting
Curr_Key	db 0			; This is it

Curr_CS		db 0			; Character set
PC_Col		db 0			; Colour

PC_X		dw 0			; X
PC_Y		dw 0			; Y

PC_Rot		db 0			; Rotate value

PC_Height	db 0			; Width (PELS)
PC_Width	db 0			; Width (PELS)

PC_iX_Cnt	db 0			; Width in bytes
PC_X_Cnt	db 0			; Width in bytes

PS_DX		dw 0			; X Step
PS_LHS		dw 0			; Left hand margin

