o		equ offset		; Silly syntax
b		equ byte ptr		;
s		equ short		;

dos		equ 21h			; DOS int
vdu		equ 10h			; VDU int

Kbd_Status	equ 64h			; Port address
Kbd_Data	equ 60h			;

Int_Ctrl	equ 20h			; Command
Int_Data	equ 21h			; Data (Masks)

EOI		equ 20h			; Command to 8259A

VDU_Status	equ 3DAh		; Bit 0 = vertical sync

CODE		segment byte public

		assume cs:CODE

; Given al,ah set the flags

Set_Frame_Flags:mov cs:Kill_FS,ah	; En/Dis frame lock
		mov cs:Int_Dis,al	; Int

		; Assert time-out if no frame-lock

		mov cs:Long_Time_F,ah	; Set if no frame sync

		ret			; Done

;  Wait for a frame

Next_Frame:	test b cs:Kill_FS,-1	; Sync ?
		jnz NF_X		; No, skip

		push dx			; Save regs
		push ax			;

		dec word ptr cs:Game_Time ; Time-out yet ?
		jnz NF_TO		; No, skip

		; Flag that we have been here a while

		mov al,1		; Set flag
		mov b cs:Long_Time_F,al	;

		; Wait a bit

NF_TO:		sti			; EI

		test b cs:Int_Dis,-1	; DI ?
		jnz NF_NI		; No, skip

		cli			; Disable ints

NF_NI: 		mov dx,VDU_Status	; Port address
		mov ah,08h		; The frame sync bit

NF_Lp:		in al,dx		; Look at VDU
		and al,ah		; Frame sync bit
		jnz NF_Lp		; Wait for end

NF_Lp1:		in al,dx		; Look at VDU
		and al,ah		; Frame sync bit
		jz NF_Lp1		; Wait for start

		sti			; Enable ints

		pop ax			; Restore
		pop dx			;

NF_X:		ret			; Done

SetUpDisplay:	mov ah,0Fh		; Get video mode
		int vdu			;

		mov cs:OldMode,al	; Save mode
		mov cs:OldPage,bh	; Save page

		mov ax,0A000h		; EGA Page
		mov cs:VDU_Seg,ax	; Save it
		mov ds,ax		;

		; Now, save the palette

		push cs			; Use the code-seg
		pop es			;

		mov dx,o DAC_Save	; Save them here

		mov ax,01017h		; Read DAC values
		mov bx,00h		; All of them
		mov cx,0FFh		;

		int vdu			; Read them

		; Now, set the new palette

		call Set_640_350	; Front-end mode

		; Done, exit

Wait_Settle:	mov cx,0		; Pause

SUD_Wait:	push ax			; Waste some time
		push ax			;
		pop ax			;
		pop ax			;

		loop SUD_Wait		; Wait for display to settle

		; Return the VDU segment

		mov ax,cs:VDU_Seg	; Return it

		clc			; OK
		ret			; Done

Kill_Vdu:	sti			; Enable ints

		mov ah,0		; Restore video mode
		mov al,cs:OldMode	;
		int vdu

		mov ah,5		; Restore page
		mov al,cs:OldPage	;
		int vdu			;

		jmp Wait_Settle		; Allow it to settle

; Set the video modes

Set_640_350:	mov al,010h		; EGA 640.350 16 colour

		mov ah,0		; Set mode
		int vdu			;

		mov dx,o DAC_Save	; New values

		push cs			; Use the code-seg
		pop es			;

		mov ax,01012h		; Write DAC values
		mov bx,00h		; All of them
		mov cx,100h		;

		int vdu			; Read them

		; Set the colours

		ret			; Done

; Set the video modes

Set_320_200:	mov al,013h		; VGA 320.200 256 colour

		mov ah,0		; Set mode
		int vdu			;

		; Set the colours

;		push cs			; Use the code-seg
;		pop es			;

;		mov dx,o Palette	; New values

;		mov ax,01012h		; Write DAC values
;		mov bx,00h		; All of them
;		mov cx,100h		;

;		int vdu			; Read them

		ret			; Done

; If called with al = NZ, then we use the AT keyboard

Link_Kbd:	mov dl,al		; Save the flag

		xor ax,ax		; Clear flags
		mov cs:Kbd_Flags,al	;

		; Clear the flags

		mov bx,o Key_Matrix	; Point at the keys
		mov cx,128		; This many

LK_Lp:		mov b cs:[bx],0		; Kill the flags
		inc bx			; Step the ptr

		loop LK_Lp		; Loop

		; Assume AT keyboard

		mov b cs:Keyb_Type,0	; Assert AT

		; Are we an AT ?

		mov ax,0F000h		; Seg = F000
		mov ds,ax		;

		mov al,ds:[0FFFEh]	; Get the byte from F000:FFFE

		cmp al,0FCh		; AT ?
		je LK_1			; Yes, skip

		; Now, are we forced to AT ?

		cmp dl,0		; Forced to AT ?
		jnz LK_1		; Yes, skip

		; Right, assume XT

		mov b cs:Keyb_Type,1	; Assert XT
		
		; Now get the vector, etc

LK_1:		push es			; Save segs
		push ds			;

		mov al,09h		; Kbd vector

		mov ah,035h		; Read vector
		int dos			;

		mov cs:Kbd_Old_Off,bx	; Save it
		mov bx,es		;
		mov cs:Kbd_Old_Seg,bx	; Save it

		push cs			; Get segment
		pop ds			;

		mov dx,o Kbd_Interrupt	; Assert it
		mov al,09h		; Keyboard int vector

		mov ah,25h		; Set interrupt vector
		int dos			;

		pop ds			; Restore
		pop es			;

		ret			; Done

Unlink_Kbd:	push ds			; Save ds

		mov dx,cs:Kbd_Old_Seg	; Get old vector
		mov ds,dx		;
		mov dx,cs:Kbd_Old_Off	;

		mov al,09h		; Kbd int

		mov ah,25h		; Restore it
		int dos			;

		pop ds			; Restore

		clc			; OK
		ret			; Done

; XT int

KI_XT:		push ax
		push bx

		in al,060h		; Read it
		mov bl,al		; Dup
		mov ah,al		; Dup

		in al,061h		; Ctrl

		or al,80h		; High
		out 61h,al		;
		and al,7Fh		; Low
		out 61h,al		;

		and bx,007Fh		; Point at the matrix
		add bx,o Key_Matrix	;

		rol ah,1		; Flag to bit 0
		not ah			; Invert state
		and ah,01h		; 0 = Not pressed, 1 = Pressed

		mov cs:[bx],ah		; Save key state

		; Exit

		mov al,61h		; Drive 8259A
		out 020h,al		;

		pop bx			; Restore
		pop ax			;

		iret			;

Kbd_Interrupt:	sti			; EI

		test b cs:Keyb_Type,1	; XT ?
		jnz KI_XT		; Yes, skip

		; AT type keyboard

		push ax			; Save regies
		push bx			;
		push cx			;
		push dx			;

		push di			;
		push si			;

		mov al,0ADh		; Disable keyboard
		call SendToKbd		; Ship it

		cli			; DI
		sub cx,cx		; Clear count

KI_Lp1:		in al,Kbd_Status	; Buffer full ?
		test al,02h		;
		loopnz KI_Lp1		; Loop

		in al,Kbd_Data		; Get the character from t'kbd
		sti			; Enable ints again

		cmp al,0FAh		; Is this an ACK ?
		je KI_Ack		; Yes, set the flag and ignore it.

		cmp al,0FEh		; Is this a resend ?
		je KI_Resend		; Yes, just ignore it (for now)

		mov bl,al		; Create ptr to Key Matrix
		and bx,007Fh		;
		add bx,o Key_Matrix	;

		rol al,1		; Flag to bit 0
		not al			; Invert state
		and al,01h		; 0 = Not pressed, 1 = Pressed

		mov cs:[bx],al		; Save key state

Kbd_I_Ret:	cli			; DI

		mov al,EOI		; End of interrupt
		out Int_Ctrl,al		; Tell 8259

		mov al,0AEh		; Enable kbd
		call SendToKbd		; Tell kbd

		cli			; DI

		pop si			; Restore regs
		pop di			;

		pop dx			;
		pop cx			;
		pop bx			;
		pop ax			;

		iret			; Done, restore flags

KI_Resend:	
KI_Ack:		or b cs:Kbd_Flags,01h	; Ack recieved
		jmp s Kbd_I_Ret		; Done

SendToKbd:	push ax			; Talk to Kbd

		cli			; DI
		sub cx,cx		; Clear count

STK_Lp:		in al,Kbd_Status	; Buffer full ?
		test al,02h		;
		loopnz STK_Lp		; Loop

		pop ax			; Restore data
		out Kbd_Status,al	; Send it

		sti			; EI
		ret			; Done

Rand_32:	PUSH cx			;
		PUSH bx			;

		CALL Rand_Inner		; Get next

		XOR al,dh		; Scramble it a bit
		XOR ah,dl		;

		POP bx			; Restore
		POP cx			;

		RET			; Done

Rand_Inner:	MOV ax,word ptr CS:Seed	; This is from Turbo Pascal
		MOV bx,word ptr CS:Seed+2;
		MOV cx,ax		;

		MUL word ptr CS:Nasty	;

		SHL cx,1		;
		SHL cx,1		;
		SHL cx,1		;

		ADD ch,cl		;
		ADD dx,cx		;
		ADD dx,bx		;

		SHL bx,1		;
		SHL bx,1		;

		ADD dx,bx		;
		ADD dh,bl		;

		MOV cl,5		;
		SHL bx,cl		;

		ADD dh,bl		;

		ADD ax,1		;
		ADC dx,0		;

		MOV word ptr CS:Seed,ax	;
		MOV word ptr CS:Seed+2,dx;

		RET			;

Nasty:		DW 33797		; Nasty number

; Fixed data here

;		include Palette.inc	; Include the palette

Game_Time	dw 70*20;0		; About 10 seconds

Seed		db "NIG!"		; Random number seed

DAC_Save	db 256*3 dup (?)	; Saved DAC registers

VDU_Seg		dw 0			; Video segment
Kbd_Old_Off	dw 0			; Saved vector
Kbd_Old_Seg	dw 0			;

Kill_FS		db 0			; No sync
Int_Dis		db 0			; No DI's

Keyb_Type	db 0			; NZ = XT

Int_Mask	db 0			; Initial int. mask

OldMode		db 0			; Initial video mode
OldPage		db 0			; Initial video page

Kbd_Flags	db 0			; Some flags

Key_Matrix	db 128 dup (0)		; Key matrix

Long_Time_F	db 0			; Not yet

CODE		ends

		public SetUpDisplay
		public Link_Kbd
		public Unlink_Kbd
		public Kill_VDU
		public Set_640_350
		public Set_320_200
		public Next_Frame
		public Set_Frame_Flags
		public Key_Matrix
		public Long_Time_F

		public Rand_32
		public Seed

		end

